import "dart:html";
import "dart:async";

import 'package:angular/angular.dart';

@Component(
  selector: 'jumbotron',
  templateUrl: 'jumbotron.html',
  cssUrl: 'jumbotron.css')
class JumbotronComponent {
  static Function _(e) => e.preventDefault();
  static const String DEFAULT_SOCKADDR = "ws://127.0.0.1:8082/";
  static const int ENTER = 13, TAB = 9, BACKSPACE = 8, UP = 38, DOWN = 40, LEFT = 37, RIGHT = 39;

  List<String> _output;
  List<String> _history;
  int _historyPos;
  String _command;
  String _cursor;
  String _ps1;
  String _socketAddress;
  WebSocket _socket;

  @NgAttr("ps1")
  set ps1(String value) { _ps1 = value; }
  get ps1 => _ps1;

  @NgAttr("sockaddr")
  set sockaddr(String address) { _socketAddress = "ws://$address/"; }
  get sockaddr => _socketAddress;

  get cursor => _cursor;

  get output {
    if (_output == null) {
      _output = new List();
    }
    return _output;
  }

  get history {
    if (_history == null) {
      _history = new List();
    }
    return _history;
  }

  set command(String value) => _command = value;
  get command => _command;

  bool get isConnected => 
    _socket != null && _socket.readyState == WebSocket.OPEN;

  void send(String msg) {
    if (isConnected) {
      _socket.send(msg);
    } else {
      write("error: no server connect established. Try running jtconn");
    }
  }

  void write(String msg) { 
    print("write called");
    output.add("$msg"); 
  }

  void reset() { output.clear();}

  void clear() => reset();

  void writeHistory() {
    for (int i=0; i<history.length; i++) {
      write(" $i ${history[i]}");
    }
  }

  void tryConnection() {
    disconnect();
    if (sockaddr == null) {
      _socket = new WebSocket(DEFAULT_SOCKADDR);
    } else {
      _socket = new WebSocket(sockaddr);
    }

    write("attempting to connect to socket ${_socket.url}...");
    _socket.onError.listen((Event e) {
      write("could not connect to server");
    });
    _socket.onOpen.listen((Event e) {
      write("server connection established!");
    });
    _socket.onMessage.listen((MessageEvent e) {
      write("${e.data}");
    });
    _socket.onClose.listen((Event e) {
      write("connection to server closed.");
    });
    return null;
  }

  void disconnect() {
    if (_socket != null) {
      _socket.close(/* need to give a code and reason */);
      _socket = null;
    }
  }

  void init() {
    window.onKeyDown.listen(_handleKeyDown);
    window.onKeyPress.listen(_handleKeyPress);
    command = "";
    _historyPos = 0;
  }

  void addToHistory(String command) {
    if (history.isEmpty || history.last.compareTo(command) != 0) {
        history.add(command);
    }
    _historyPos = history.length;
  }

  void _handleEnterPressed() {
    write("$ps1${command}");
    switch(command) {
      case "clear": clear(); break;
      case "reset": reset(); break;
      case "history": writeHistory(); break;
      case "jtconn": tryConnection(); break;
      case "jtdisconn": disconnect(); break;
      case "jtset": setNgAttribute(command); break;
      default: send(command);
    }
    addToHistory(command);
    command = "";
  }

  void _handleTabPressed() {
    send("\\t");
  }

  void _handleBackspacePressed() {
    if (command.length > 0) {
      command = command.substring(0, command.length-1);
    }
  }

  void _handleUpPressed() {
    if (_historyPos > 0) {
      command = history[--_historyPos];
    }
  }

  void _handleDownPressed() {
    if (_historyPos < history.length) {
      ++_historyPos;
      command = _historyPos == history.length
        ? ""
        : history[_historyPos];
    }
  }

  void _handleKeyDown(KeyboardEvent e) {
    //print("keydown: ${e.keyCode}");
    switch(e.keyCode) {
      case ENTER:     _(e); _handleEnterPressed(); break;
      case TAB:       _(e); _handleTabPressed(); break;
      case BACKSPACE: _(e); _handleBackspacePressed(); break;
      case UP:        _(e); _handleUpPressed(); break;
      case DOWN:      _(e); _handleDownPressed(); break;
    }
  }

  void _handleKeyPress(KeyboardEvent e) {
    //print("keypress: ${e.charCode}");
    switch(e.keyCode) {
      case ENTER: break;
      case TAB: break;
      case BACKSPACE: break;
      default:
        _(e);
        command += new String.fromCharCode(e.charCode);
    }
  }

  JumbotronComponent() {
    init();
    tryConnection();
  }
}